{
  "columns": [
    {
      "aid": "54",
      "attribute": "*"
    }
  ],
  "where": [
    {
      "condition": {
        "aid": "54",
        "attribute": "Name",
        "operator": "OP_CI_EQ",
        "stringArray": {
          "values": [
            "s"
          ]
        }
      }
    }
  ]
}